package com.tricomgroup.tools.ejb;

import java.io.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.lang.reflect.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.ejb.deployment.*;

public class EJBSkeleton {

 public static void main(String args[]) {
  new EJBSkeletonFrame();
 }
}

class EJBSkeletonFrame extends JFrame implements ActionListener, ItemListener, EJBSkeletonConstants {
 JMenuBar mb = new JMenuBar();
 ButtonGroup ejbVersion = new ButtonGroup();
 ButtonGroup ejbType = new ButtonGroup();
 ButtonGroup sessionType = new ButtonGroup();
 ButtonGroup entityType = new ButtonGroup();
 JRadioButton versionOneZero = new JRadioButton("1.0",true);
 JRadioButton versionOneOne = new JRadioButton("1.1",false);
 JRadioButton session = new JRadioButton("Session",true);
 JRadioButton entity = new JRadioButton("Entity",false);
 JRadioButton stateless = new JRadioButton("Stateless",true);
 JRadioButton stateful = new JRadioButton("Stateful",false);
 JRadioButton containerManaged = new JRadioButton("Container Managed",true);
 JRadioButton beanManaged = new JRadioButton("Bean Managed",false);
 JCheckBox compile = new JCheckBox("Compile EJB Source Files");
 JCheckBox jar = new JCheckBox("Package Bean");
 JCheckBox client = new JCheckBox("Generate Client Source");
 JButton generate = new JButton("Generate");
 JButton source = new JButton("Add EJB Source Code...");
 JButton close = new JButton("Close");
 JButton about = new JButton("About...");
 JTextField pkg = new JTextField(20);
 JTextField base = new JTextField(20);
 JTextField root = new JTextField(20);
 JPanel northPanel = new JPanel();
 JPanel northPanelA = new JPanel();
 JPanel northPanelB = new JPanel();
 JPanel cardPanelA = new JPanel();
 JPanel cardPanelB = new JPanel();
 JPanel northPanelC = new JPanel();
 JPanel southPanel = new JPanel();
 JPanel centerPanel = new JPanel();
 JPanel eastPanel = new JPanel();
 CardLayout cl = new CardLayout();
 String newLine = System.getProperty("line.separator");
 String baseName;
 String pkgName;
 String rootName;
 String destPrefix;
 ArrayList ejbSource;
 ArrayList cmpFieldSource;
 ArrayList variableFieldSource;
 ArrayList pkFieldSource;
 ProgressDialog pd;
 Process p;

 EJBSkeletonFrame() {
  super("EJBSkeleton");

  /* Components should be added to the container's content pane */
  Container cp = getContentPane();

  /* Add radio buttons to corresponding group */
  ejbVersion.add(versionOneZero);
  ejbVersion.add(versionOneOne);

  ejbType.add(session);
  ejbType.add(entity);

  sessionType.add(stateless);
  sessionType.add(stateful);

  entityType.add(containerManaged);
  entityType.add(beanManaged);

  /* Set the north panel layout */
  northPanel.setLayout(new BorderLayout());

  /* Set the north sub-panel layout */
  northPanelA.setLayout(new GridLayout(3,2));

  /* Add components to the north sub-panels */
  northPanelA.add(new JLabel("Root Package Directory:"));
  northPanelA.add(root);
  northPanelA.add(new JLabel("Package Name:"));
  northPanelA.add(pkg);
  northPanelA.add(new JLabel("Base Name:"));
  northPanelA.add(base);

  northPanelB.setLayout(new GridLayout(2,3));
  northPanelB.add(new JLabel("EJB Version:"));
  northPanelB.add(versionOneZero);
  northPanelB.add(versionOneOne);
  northPanelB.add(new JLabel("EJB Type:"));
  northPanelB.add(session);
  northPanelB.add(entity);

  northPanelC.setLayout(cl);
  northPanelC.add(cardPanelA,"cardA");
  northPanelC.add(cardPanelB,"cardB");

  /* Set the card panel layouts */
  cardPanelA.setLayout(new GridLayout(1,3));
  cardPanelB.setLayout(new GridLayout(1,3));

  /* Add the components the cards */
  cardPanelA.add(new JLabel("EJB Sub-type:"));
  cardPanelA.add(stateless);
  cardPanelA.add(stateful);

  cardPanelB.add(new JLabel("EJB Sub-type:"));
  cardPanelB.add(beanManaged);
  cardPanelB.add(containerManaged);

  northPanel.add(BorderLayout.NORTH,northPanelA);
  northPanel.add(BorderLayout.CENTER,northPanelB);
  northPanel.add(BorderLayout.SOUTH,northPanelC);

  /* Add components to the south panel */
  southPanel.add(generate);
  southPanel.add(source);
  southPanel.add(close);
  southPanel.add(about);

  centerPanel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Miscellaneous Options"),BorderFactory.createEmptyBorder(10,10,10,10)));
  centerPanel.add(client);
  centerPanel.add(compile);
  centerPanel.add(jar);

  /* Add panels to frame */
  cp.add(BorderLayout.NORTH,northPanel);
  cp.add(BorderLayout.CENTER,centerPanel);
  cp.add(BorderLayout.SOUTH,southPanel);

  /* Set new documents */
  base.setDocument(new EnableGenerateDocument());
  pkg.setDocument(new EnableGenerateDocument());

  /* Add the item listeners */
  session.addItemListener(this);
  entity.addItemListener(this);
  compile.addItemListener(this);

  /* Add the action listeners */
  generate.addActionListener(this);
  source.addActionListener(this);
  close.addActionListener(this);
  about.addActionListener(this);

  /* Add the window listener */
  addWindowListener(new WindowAdapter() {
   public void windowClosing(WindowEvent evt) {
    dispose(); deleteTempFiles(); System.exit(0);}});

  /* Disable Generate button, Add EJB Source Code button, and Package Bean checkbox */
  generate.setEnabled(false);
  source.setEnabled(false);
  jar.setEnabled(false);

  /* Size the frame */
  pack();

  /* Set the frame un-sizable */
  setResizable(false);

  /* Center the frame */
  Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
  Rectangle frameDim = getBounds();
  setLocation((screenDim.width - frameDim.width) / 2,(screenDim.height - frameDim.height) / 2);

  /* Show the frame */
  setVisible(true);
 }

 class EnableGenerateDocument extends PlainDocument
 {
   public void insertString(int offset, String str, AttributeSet set) throws BadLocationException
   {
    int strLen = str.length();
    int len = getLength();

    super.insertString(offset,str,set);
    updateGenerateButton();
   }

   public void remove(int offset, int len) throws BadLocationException
   {
    super.remove(offset,len);
    updateGenerateButton();
   }

   public void updateGenerateButton()
   {
    int baseLen = base.getText().length();
    int pkgLen = pkg.getText().length();

    if (baseLen > 0 && pkgLen > 0 && !generate.isEnabled())
    {
      generate.setEnabled(true);
      source.setEnabled(true);
    }
    else if (baseLen == 0 || pkgLen == 0 && generate.isEnabled())
    {
      generate.setEnabled(false);
      source.setEnabled(false);
    }
   }
 }

 public void actionPerformed(ActionEvent evt) {
  Object obj = evt.getSource();

  if (obj == generate)
  {
   SwingUtilities.invokeLater(new Runnable() {
    public void run() {
     setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

     pd = new ProgressDialog(EJBSkeletonFrame.this);

     baseName = base.getText();
     pkgName = pkg.getText();
     rootName = root.getText();
     destPrefix = "src/" + (versionOneZero.isSelected() ? "v1.0/" : "v1.1/");

     if (generateEJBHome() & generateEJB() & generateEJBRemote() &
         (entity.isSelected() ? generatePrimaryKeyClass() : true))
     {
      pd.addMessage("Successfully created EJB source files!" + newLine);

      /* Compile source files */
      if (compile.isSelected())
      {
       BufferedReader br1 = null;
       BufferedReader br2 = null;
       String str = null;
       String msg = "";

       try
       {
         p = Runtime.getRuntime().exec("javac " + (rootName.length() != 0 ? "-d " + rootName + " " : "-d . ") + destPrefix + baseName + "/" + baseName + "*.java");

         br1 = new BufferedReader(new InputStreamReader(p.getErrorStream()));

         while ((str = br1.readLine()) != null)
          msg += str + newLine;

         if (msg.length() == 0)
          pd.addMessage("Successfully compiled EJB source files!" + newLine);
         else
          pd.addMessage(msg);

         if (jar.isSelected() && msg.length() == 0)
         {
           if (versionOneZero.isSelected() ? generateEJB10DeploymentDescriptor() && generateEJB10DeploymentDescriptorManifest(): generateEJB11DeploymentDescriptor())
           {
             String rootPrefix = rootName.length() != 0 ? rootName.replace('\\','/') : ".";
             String pkgPrefix = pkgName.replace('.','/');
             String filePrefix = destPrefix + baseName + "/jar/";
             String fileSuffix = "BeanJar.jar -C ";
             String execStr = null;
             File f = new File(filePrefix);

             pd.addMessage("Successfully created EJB deployment descriptor!" + newLine);

             if (!f.exists())
              f.mkdirs();

             if (versionOneZero.isSelected())
              execStr = new StringBuffer(100).append("jar -cmf ").
                                              append(destPrefix).
                                              append(baseName).
                                              append("/").
                                              append(baseName).
                                              append("Manifest ").
                                              append(filePrefix).
                                              append(baseName).
                                              append(fileSuffix).
                                              append(rootPrefix).
                                              append(" ").
                                              append(pkgPrefix).toString();
             else
              execStr = new StringBuffer(100).append("jar -cf ").
                                              append(filePrefix).
                                              append(baseName).
                                              append(fileSuffix).
                                              append(rootPrefix).
                                              append(" ").
                                              append(pkgPrefix).
                                              append(" -C ").
                                              append(destPrefix).
                                              append(baseName).
                                              append(" META-INF").toString();

             p = Runtime.getRuntime().exec(execStr);

             br2 = new BufferedReader(new InputStreamReader(p.getErrorStream()));

             while ((str = br2.readLine()) != null)
              msg += str + newLine;

             if (msg.length() == 0)
              pd.addMessage("Successfully packaged EJB class files and deployment descriptor!" + newLine);
             else
              pd.addMessage(msg);
           }
         }

         if (client.isSelected() && msg.length() == 0)
          if (generateClientSource())
           pd.addMessage("Successfully created client source file!");
       }
       catch(IOException ioe)
       {
         pd.addMessage(ioe.getMessage());
       }
       finally
       {
         try
         {
          if (br1 != null)
           br1.close();

          if (br2 != null)
           br2.close();
         }
         catch(IOException ioe)
         {
         }
       }
      }

     }

     /* Change the cursor back to the default */
     setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

     pd.setVisible(true);
    }
   });
  }
  else if (obj == close)
   dispatchEvent(new WindowEvent(this,WindowEvent.WINDOW_CLOSING));
  else if (obj == about)
   JOptionPane.showMessageDialog(this,"<html><center>EJBSkeleton - An EJB Skeleton Code Generator<p>Pat Paternostro, Tri-Com Consulting Group, L.L.C.</center></html>","About EJBSkeleton",JOptionPane.INFORMATION_MESSAGE);
  else if (obj == source)
   new SourceCodeDialog(this);
 }

 private void deleteTempFiles()
 {
   File f = new File(".");

   File[] tempFiles = f.listFiles(new FilenameFilter() {
     public boolean accept(File dir, String name) {
       return (name.startsWith(TEMP_FILE_PREFIX) && (name.endsWith("Fields._tmp") || name.endsWith("Source._tmp")));
     }
   });

   for (int i = 0; i < tempFiles.length; i++)
    tempFiles[i].delete();
 }

 public void itemStateChanged(ItemEvent evt)
 {
   Object obj = evt.getSource();

   if (obj == session)
    cl.show(northPanelC,"cardA");
   else if (obj == entity)
    cl.show(northPanelC,"cardB");
   else if (obj == compile)
   {
     if (compile.isSelected())
      jar.setEnabled(true);
     else
     {
      if (jar.isSelected())
       jar.setSelected(false);

      jar.setEnabled(false);
     }
   }
 }

 private boolean generateEJBHome()
 {
   File f = new File(destPrefix + baseName);
   PrintWriter pw = null;
   boolean bReturn = true;

   try
   {
     if (!f.exists())
      f.mkdirs();

     pw = new PrintWriter(new FileWriter(destPrefix + baseName + "/" + baseName + "Home.java"));

     writePackageStatement(pw);
     pw.println("import " + pkgName + ".*;");
     writeImportStatements(pw,false);

     pw.println("public interface " + baseName + "Home extends EJBHome {");
     pw.println(" public " + baseName + " create() throws RemoteException, CreateException;");

     if (entity.isSelected())
      pw.println(" public " + baseName + " findByPrimaryKey(" + baseName + "PK pk) throws RemoteException, FinderException;");

     pw.println("}");
   }
   catch(IOException ioe)
   {
     bReturn = false;
     pd.addMessage("Could not create EJB home interface source file!" + newLine);
   }
   finally
   {
     if (pw != null)
      pw.close();
   }

   return bReturn;
 }

 private boolean generateEJBRemote()
 {
   ArrayList methods = null;
   PrintWriter pw = null;
   boolean bReturn = true;

   try
   {
     pw = new PrintWriter(new FileWriter(destPrefix + baseName + "/" + baseName + ".java"));

     writePackageStatement(pw);
     writeImportStatements(pw,false);
     pw.println("public interface " + baseName + " extends EJBObject {");
     generateEJBRemoteMethodDeclarations(pw);
     pw.println("}");
   }
   catch(IOException ioe)
   {
     bReturn = false;
     pd.addMessage("Could not create EJB remote interface source file!");
   }
   finally
   {
     if (pw != null)
      pw.close();
   }

   return bReturn;
 }

 private boolean generateEJB()
 {
   PrintWriter pw = null;
   boolean entityBean = entity.isSelected();
   boolean bReturn = true;
   int size = 0;

   try
   {
     pw = new PrintWriter(new FileWriter(destPrefix + baseName + "/" + baseName + "Bean.java"));

     writePackageStatement(pw);

     if (entity.isSelected())
      if (versionOneOne.isSelected() || (versionOneZero.isSelected() && beanManaged.isSelected()))
       pw.println("import " + pkgName + ".*;");

     writeImportStatements(pw,true);

     pw.println("public class " + baseName + "Bean implements " + (entityBean ? "EntityBean" : "SessionBean") + " {");

     if (entityBean)
     {
       pw.println(" private EntityContext ctx;");

       /* Read the temp instance/class field(s) file and include its contents in the bean source */
       variableFieldSource = readTempFile(TEMP_FILE_PREFIX + baseName + TEMP_FIELDS_SUFFIX);
       writeContents(variableFieldSource,pw);

       /* Read the temp PK field(s) file */
       pkFieldSource = readTempFile(TEMP_FILE_PREFIX + baseName + TEMP_PK_FIELDS_SUFFIX);

       if (containerManaged.isSelected())
       {
        /*
         * The fields that make up a compound primary key are required to be defined, with matching
         * identifiers and types, in the container-managed entity bean class otherwise the bean will
         * not pass verification.
         */
         writeContents(pkFieldSource,pw);

         /* Read the temp CMP field(s) file and include its contents in the bean source */
         cmpFieldSource = readTempFile(TEMP_FILE_PREFIX + baseName + TEMP_CMP_FIELDS_SUFFIX);
         writeContents(cmpFieldSource,pw);
       }

       pw.println();
     }
     else
     {
      pw.println(" private SessionContext ctx;" + newLine);

      if (stateful.isSelected())
      {
       /* Read the temp instance/class field(s) file and include its contents in the bean source */
       variableFieldSource = readTempFile(TEMP_FILE_PREFIX + baseName + TEMP_FIELDS_SUFFIX);
       writeContents(variableFieldSource,pw);
      }
     }

    /*
     * Version 1.0 and 1.1 bean-managed persistence beans are required to return the primary key on
     * create methods. Version 1.1 container-managed persistence beans are required to return null
     * and version 1.0 container-managed persistence beans return void.
     */
     pw.println(" public " + (entityBean && (beanManaged.isSelected() || ((versionOneOne.isSelected() && containerManaged.isSelected()))) ? baseName + "PK" : "void") + " ejbCreate() throws CreateException" + newLine + " {" + (entityBean && beanManaged.isSelected() ? newLine + "  return new " + baseName + "PK();" : (entityBean && containerManaged.isSelected() && versionOneOne.isSelected() ? newLine + "  return null;" : "")) + newLine + " }" + newLine);
     pw.println(" public void ejbRemove()" + newLine + " {" + newLine + " }" + newLine);
     pw.println(" public void ejbPassivate()" + newLine + " {" + newLine + " }" + newLine);
     pw.println(" public void ejbActivate()" + newLine + " {" + newLine + " }" + newLine);

     if (entityBean)
     {
     /*
      * Bean-managed persistence beans are required to provide an implementation for all the home
      * interface finder methods (e.g., ejbFindByPrimaryKey()). Container-managed persistence beans
      * must not provide an implementation for the home interface finder methods.
      */
      if (beanManaged.isSelected())
       pw.println(" public " + baseName + "PK ejbFindByPrimaryKey(" + baseName + "PK pk)" + newLine + " {" + newLine + "  return new " + baseName + "PK();" + newLine + " }" + newLine);

      pw.println(" public void ejbPostCreate()" + newLine + " {" + newLine + " }" + newLine);
      pw.println(" public void ejbStore()" + newLine + " {" + newLine + " }" + newLine);
      pw.println(" public void ejbLoad()" + newLine + " {" + newLine + " }" + newLine);
      pw.println(" public void setEntityContext(EntityContext ctx)" + newLine + " {" + newLine + "  this.ctx = ctx;" + newLine + " }" + newLine);
      pw.println(" public void unsetEntityContext()" + newLine + " {" + newLine + "  ctx = null;" + newLine + " }" + newLine);
     }
     else
      pw.println(" public void setSessionContext(SessionContext ctx)" + newLine + " {" + newLine + "  this.ctx = ctx;" + newLine + " }" + newLine);

     /* Read the temp EJB source file and include its contents in the bean source */
     ejbSource = readTempFile(TEMP_FILE_PREFIX + baseName + TEMP_SOURCE_SUFFIX);
     writeContents(ejbSource,pw);

     pw.println("}");
   }
   catch(IOException ioe)
   {
     bReturn = false;
     pd.addMessage("Could not create EJB class source file!" + newLine);
   }
   finally
   {
     if (pw != null)
      pw.close();
   }

   return bReturn;
 }

 private void generateEJBRemoteMethodDeclarations(PrintWriter pw)
 {
   int size = ejbSource.size();
   int index = 0;

   for (int i = 0; i < size; i++)
   {
     String str = (String) ejbSource.get(i);

     if ((index = str.indexOf("public")) != -1)
      pw.println(" " + str.substring(index,str.indexOf(')') + 1).trim() + " throws RemoteException;");
   }
 }

 private boolean generatePrimaryKeyClass()
 {
   PrintWriter pw = null;
   boolean bReturn = true;

   try
   {
     pw = new PrintWriter(new FileWriter(destPrefix + baseName + "/" + baseName + "PK.java"));

     writePackageStatement(pw);

     pw.println("import java.io.*;" + newLine);
     pw.println("public class " + baseName + "PK implements Serializable {");

     /* Write the contents of the temp PK field(s) file that were read earlier */
     writeContents(pkFieldSource,pw);

     pw.println();
     pw.println(" public int hashCode()" + newLine + " {" + newLine + "  return 0;" + newLine + " }" + newLine);
     pw.println(" public boolean equals(Object obj)" + newLine + " {" + newLine + "  return false;" + newLine + " }");
     pw.println("}");
   }
   catch(IOException ioe)
   {
     bReturn = false;
     JOptionPane.showMessageDialog(this,"Could not create EJB primary key class source file!","Error",JOptionPane.ERROR_MESSAGE);
     pd.addMessage("Could not create EJB primary key class source file!" + newLine);
   }
   finally
   {
     if (pw != null)
      pw.close();
   }

   return bReturn;
 }

 private boolean generateEJB10DeploymentDescriptor()
 {
   DeploymentDescriptor dd = null;
   FileOutputStream fos = null;
   ObjectOutputStream oos = null;
   boolean entityBean = entity.isSelected();
   boolean bReturn = true;
   String packagePrefix = pkgName + "." + baseName;
   String pathName = (rootName.length() != 0 ? rootName.replace('\\','/') + "/" : rootName) + pkgName.replace('.','/') + "/" + baseName;

   if (entityBean)
    dd = new EntityDescriptor();
   else
    dd = new SessionDescriptor();

   /* Set bean, home, and remote names */
   dd.setEnterpriseBeanClassName(packagePrefix + "Bean");
   dd.setHomeInterfaceClassName(packagePrefix + "Home");
   dd.setRemoteInterfaceClassName(packagePrefix);

   if (entityBean)
   {
     ((EntityDescriptor) dd).setPrimaryKeyClassName(packagePrefix + "PK");
     ((EntityDescriptor) dd).setReentrant(false);

     if (containerManaged.isSelected())
     {
       try
       {
         /* Use a custom class loader to prevent caching that occurs with the Class.forName() method */
         Class beanClass = new FileClassLoader().loadClassBytes(new File(pathName + "Bean.class"));
         int pkSize = pkFieldSource.size();
         int cmpSize = cmpFieldSource.size();
         Field[] persistentFields = new Field[pkSize + cmpSize];
         int i = 0;
         int j = 0;

         for (i = 0; i < pkSize; i++)
         {
           String str = (String) pkFieldSource.get(i);
           int index = 0;

           if ((index = str.lastIndexOf(' ')) != -1)
            persistentFields[i] = beanClass.getDeclaredField(str.substring(index + 1,str.length() - 1));
         }

         for (j = i, i = 0; i < cmpSize; i++, j++)
         {
           String str = (String) cmpFieldSource.get(i);
           int index = 0;

           if ((index = str.lastIndexOf(' ')) != -1)
            persistentFields[j] = beanClass.getDeclaredField(str.substring(index + 1,str.length() - 1));
         }

         ((EntityDescriptor) dd).setContainerManagedFields(persistentFields);

       }
       catch(ClassNotFoundException cnfe)
       {
         bReturn = false;
         pd.addMessage("Bean class not found! Check that your CLASSPATH includes the root package directory!" + newLine);
       }
       catch(NoSuchFieldException nsfe)
       {
         bReturn = false;
         pd.addMessage("Field not defined in bean class!" + newLine);
       }
     }
   }
   else
   {
     /* Use container-specfic timeout */
     ((SessionDescriptor) dd).setSessionTimeout(0);

     /* Set session type */
     ((SessionDescriptor) dd).setStateManagementType(stateless.isSelected() ? SessionDescriptor.STATELESS_SESSION : SessionDescriptor.STATEFUL_SESSION);
   }

   /* Set the transaction attribute */
   ControlDescriptor cd = new ControlDescriptor();
   cd.setMethod(null);
   cd.setTransactionAttribute(ControlDescriptor.TX_REQUIRED);
   dd.setControlDescriptors(new ControlDescriptor[] {cd});

   try
   {
     if (bReturn)
     {
       fos = new FileOutputStream(pathName + "DD.ser");
       oos = new ObjectOutputStream(fos);

       if (entityBean)
        oos.writeObject((EntityDescriptor) dd);
       else
        oos.writeObject((SessionDescriptor) dd);
     }
   }
   catch(IOException ioe)
   {
     bReturn = false;
     pd.addMessage("Could not create EJB 1.0 deployment descriptor!" + newLine);
   }
   finally
   {
    try
    {
      if (oos != null)
       oos.close();
    }
    catch(IOException ioe)
    {
    }
    try
    {
      if (fos != null)
       fos.close();
    }
    catch(IOException ioe)
    {
    }
   }

   return bReturn;
 }

 private boolean generateEJB10DeploymentDescriptorManifest()
 {
   PrintWriter pw = null;
   boolean bReturn = true;

   try
   {
     pw = new PrintWriter(new FileWriter(destPrefix + baseName + "/" + baseName + "Manifest"));

     pw.println("Name: " + pkgName.replace('.','/') + "/" + baseName + "DD.ser");
     pw.println("Enterprise-Bean: True");
   }
   catch(IOException ioe)
   {
     bReturn = false;
     pd.addMessage("Could not create EJB 1.0 deployment descriptor manifest!" + newLine);
   }
   finally
   {
     if (pw != null)
      pw.close();
   }

   return bReturn;
 }

 private boolean generateEJB11DeploymentDescriptor()
 {
   PrintWriter pw = null;
   boolean entityBean = entity.isSelected();
   boolean bReturn = true;
   String packagePrefix = pkgName + "." + baseName;

   try
   {
     File f = new File(destPrefix + baseName + "/META-INF");

     /* Determine if the META-INF directory exists */
     if (!f.exists())
      f.mkdirs();

     pw = new PrintWriter(new FileWriter(destPrefix + baseName + "/META-INF/ejb-jar.xml"));

     pw.println("<?xml version=\"1.0\" encoding=\"Cp1252\"?>");
     pw.println();
     pw.println("<!DOCTYPE ejb-jar PUBLIC '-//Sun Microsystems, Inc.//DTD Enterprise JavaBeans 1.1//EN' 'http://java.sun.com/j2ee/dtds/ejb-jar_1_1.dtd'>");
     pw.println();
     pw.println("<ejb-jar>");
     pw.println(" <display-name>" + baseName + "BeanJar</display-name>");
     pw.println(" <enterprise-beans>");
     pw.println(entityBean ? "  <entity>" : "  <session>");
     pw.println("   <display-name>" + baseName + "Bean</display-name>");
     pw.println("   <ejb-name>" + baseName + "Bean</ejb-name>");
     pw.println("   <home>" + packagePrefix + "Home</home>");
     pw.println("   <remote>" + packagePrefix + "</remote>");
     pw.println("   <ejb-class>" + packagePrefix + "Bean</ejb-class>");

     if (entityBean)
     {
      pw.println("   <persistence-type>" + (containerManaged.isSelected() ? "Container" : "Bean") + "</persistence-type>");
      pw.println("   <prim-key-class>" + packagePrefix + "PK" + "</prim-key-class>");
      pw.println("   <reentrant>False</reentrant>");

      if (containerManaged.isSelected())
      {
       add11CmpFields(pkFieldSource,pw);
       add11CmpFields(cmpFieldSource,pw);
      }
     }
     else
     {
      pw.println("   <session-type>" + (stateless.isSelected() ? "Stateless" : "Stateful") + "</session-type>");
      pw.println("   <transaction-type>Container</transaction-type>");
     }

     pw.println(entityBean ? "  </entity>" : "  </session>");
     pw.println(" </enterprise-beans>");

     /*
      * Set the transaction attribute for remote interface methods (session and entity beans).
      * The J2EE reference implementation deploy tool will generate errors if the method-intf
      * attribute is not used.
      */
     pw.println(" <assembly-descriptor>");
     pw.println("  <container-transaction>");
     pw.println("   <method>");
     pw.println("    <ejb-name>" + baseName + "Bean</ejb-name>");
     pw.println("    <method-intf>Remote</method-intf>");
     pw.println("    <method-name>*</method-name>");
     pw.println("   </method>");
     pw.println("   <trans-attribute>Required</trans-attribute>");
     pw.println("  </container-transaction>");

     /*
      * Set the transaction attribute for home interface methods (entity beans only).
      * The J2EE reference implementation deploy tool will generate errors if the
      * method-intf attribute is not used (as above) and will generate errors if the
      * asterisk in the method-name attribute is used multiple times. Therefore it is
      * required to explicitly code each method in the home interface to bypass the errors.
      */
     if (entityBean)
     {
      pw.println("  <container-transaction>");
      pw.println("   <method>");
      pw.println("    <ejb-name>" + baseName + "Bean</ejb-name>");
      pw.println("    <method-intf>Home</method-intf>");
      pw.println("    <method-name>create</method-name>");
      pw.println("    <method-params />");
      pw.println("   </method>");
      pw.println("   <method>");
      pw.println("    <ejb-name>" + baseName + "Bean</ejb-name>");
      pw.println("    <method-intf>Home</method-intf>");
      pw.println("    <method-name>findByPrimaryKey</method-name>");
      pw.println("    <method-params>");
      pw.println("     <method-param>" + packagePrefix + "PK</method-param>");
      pw.println("    </method-params>");
      pw.println("   </method>");
      pw.println("   <method>");
      pw.println("    <ejb-name>" + baseName + "Bean</ejb-name>");
      pw.println("    <method-intf>Home</method-intf>");
      pw.println("    <method-name>remove</method-name>");
      pw.println("    <method-params>");
      pw.println("     <method-param>java.lang.Object</method-param>");
      pw.println("    </method-params>");
      pw.println("   </method>");
      pw.println("   <method>");
      pw.println("    <ejb-name>" + baseName + "Bean</ejb-name>");
      pw.println("    <method-intf>Home</method-intf>");
      pw.println("    <method-name>remove</method-name>");
      pw.println("    <method-params>");
      pw.println("     <method-param>javax.ejb.Handle</method-param>");
      pw.println("    </method-params>");
      pw.println("   </method>");
      pw.println("   <trans-attribute>Required</trans-attribute>");
      pw.println("  </container-transaction>");
     }

     pw.println(" </assembly-descriptor>");

     pw.println("</ejb-jar>");
   }
   catch(IOException ioe)
   {
     bReturn = false;
     pd.addMessage("Could not create EJB 1.1 deployment descriptor!" + newLine);
   }
   finally
   {
     if (pw != null)
      pw.close();
   }

   return bReturn;
 }

 private boolean generateClientSource()
 {
   File f = new File(destPrefix + baseName + "/client");
   boolean bReturn = true;
   PrintWriter pw = null;

   try
   {
     if (!f.exists())
      f.mkdirs();

     pw = new PrintWriter(new FileWriter(destPrefix + baseName + "/client/" + baseName + "Client.java"));

     pw.println("import javax.rmi.*;");
     pw.println("import javax.naming.*;");
     pw.println("import " + pkgName + ".*;");
     pw.println();
     pw.println("public class " + baseName + "Client {");
     pw.println(" public static void main(String args[]) throws Throwable {");
     pw.println("  Context ctx = new InitialContext();");
     pw.println("  " + baseName + "Home home = (" + baseName + "Home) " + (versionOneOne.isSelected() ? "PortableRemoteObject.narrow(" : "") + "ctx.lookup(\"" + baseName + "Home\")" + (versionOneOne.isSelected() ? "," + baseName + "Home.class);" : ";"));
     pw.println("  " + baseName + " remote = home.create();");
     pw.println(" }");
     pw.println("}");
   }
   catch(IOException ioe)
   {
     bReturn = false;
     pd.addMessage("Could not create client source file!" + newLine);
   }
   finally
   {
     if (pw != null)
      pw.close();
   }

   return bReturn;
 }

 private void writeContents(ArrayList al, PrintWriter pw)
 {
   int size = al.size();

   for (int i = 0; i < size; i++)
    pw.println(" " + al.get(i));
 }

 private ArrayList readTempFile(String name)
 {
   BufferedReader br = null;
   ArrayList al = new ArrayList();

   /* Read the contents of the temporary file */
   try
   {
     br = new BufferedReader(new FileReader(name));
     String str = null;

     while ((str = br.readLine()) != null)
      al.add(str);
   }
   catch(FileNotFoundException fnfe)
   {
     /* Ignore the exception */
   }
   catch(IOException ioe)
   {
   }
   finally
   {
     try
     {
       if (br != null)
        br.close();
     }
     catch(IOException ioe)
     {
     }
   }

   return al;
 }

 private void add11CmpFields(ArrayList al, PrintWriter pw)
 {
   int size = al.size();

   for (int i = 0; i < size; i++)
   {
     String str = (String) al.get(i);
     int index = 0;

     if ((index = str.lastIndexOf(' ')) != -1)
      pw.println("   <cmp-field><field-name>" + str.substring(index + 1,str.length() - 1) + "</field-name></cmp-field>");
   }
 }

 private void writePackageStatement(PrintWriter pw)
 {
   pw.println("package " + pkgName + ";" + newLine);
 }

 private void writeImportStatements(PrintWriter pw, boolean isEJB)
 {
   if (!isEJB)
    pw.println("import java.rmi.*;");

   pw.println("import javax.ejb.*;" + newLine);
 }
}

class SourceCodeDialog extends JDialog implements ActionListener, EJBSkeletonConstants
{
  JTextArea sourceTextArea = new JTextArea();
  JScrollPane sourceScrollPane = new JScrollPane(sourceTextArea,ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
  JTextArea variableTextArea = new JTextArea();
  JScrollPane variableScrollPane = new JScrollPane(variableTextArea,ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
  JTextArea primaryKeyTextArea = new JTextArea();
  JScrollPane primaryKeyScrollPane = new JScrollPane(primaryKeyTextArea,ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
  JTextArea cmpTextArea = new JTextArea();
  JScrollPane cmpScrollPane = new JScrollPane(cmpTextArea,ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
  JButton save = new JButton("Save");
  JButton cancel = new JButton("Cancel");
  JPanel center = new JPanel();
  JPanel centerA = new JPanel();
  JPanel centerB = new JPanel();
  JPanel centerC = new JPanel();
  JPanel centerD = new JPanel();
  JPanel south = new JPanel();
  EJBSkeletonFrame owner;

  public SourceCodeDialog(Frame owner)
  {
    super(owner,"Add EJB Source Code",true);

    /* Save parent frame to instance variable */
    this.owner = (EJBSkeletonFrame) owner;

    /* Components should be added to the container's content pane */
    Container cp = getContentPane();

    /* Add action listeners */
    save.addActionListener(this);
    cancel.addActionListener(this);

    /* Add the window listener */
    addWindowListener(new WindowAdapter() {
     public void windowClosing(WindowEvent evt) {
      dispose();}});

    /* Set a new document to the text areas */
    cmpTextArea.setDocument(new EnableSaveDocument());
    variableTextArea.setDocument(new EnableSaveDocument());
    primaryKeyTextArea.setDocument(new EnableSaveDocument());
    sourceTextArea.setDocument(new EnableSaveDocument());

    /* Disable the Save button */
    save.setEnabled(false);

    /* Set center panel layout */
    if (this.owner.entity.isSelected())
    {
     if (this.owner.containerManaged.isSelected())
      center.setLayout(new GridLayout(4,1));
     else
      center.setLayout(new GridLayout(3,1));

     /* Set the A center panel layout and border */
     centerA.setLayout(new GridLayout(1,1));
     centerA.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Primary Key Field(s) (must be public)"),BorderFactory.createEmptyBorder(10,10,10,10)));

     /* Add component to the A center panel */
     centerA.add(primaryKeyScrollPane);

     /* Add component to the center panel */
     center.add(centerA);

     if (this.owner.containerManaged.isSelected())
     {
       /* Set the B center panel layout and border */
       centerB.setLayout(new GridLayout(1,1));
       centerB.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Container Managed Persistence Field(s) (must be public)"),BorderFactory.createEmptyBorder(10,10,10,10)));

       /* Add component to the B center panel */
       centerB.add(cmpScrollPane);

       /* Add component to the center panel */
       center.add(centerB);
     }

     layoutCCenterPanel();
    }
    else
    {
      if (this.owner.stateful.isSelected())
      {
       center.setLayout(new GridLayout(2,1));

       layoutCCenterPanel();
      }
      else
       center.setLayout(new GridLayout(1,1));
    }

    /* Set the D center panel layout and border */
    centerD.setLayout(new GridLayout(1,1));
    centerD.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("EJB Source Code"),BorderFactory.createEmptyBorder(10,10,10,10)));

    /* Add component to the D center panel */
    centerD.add(sourceScrollPane);

    /* Add component to the center panel */
    center.add(centerD);

    /* Add components to the south panel */
    south.add(save);
    south.add(cancel);

    /* Add panels to the dialog */
    cp.add(BorderLayout.CENTER,center);
    cp.add(BorderLayout.SOUTH,south);

    /* Size the dialog */
    setSize(420,480);

    /* Set the dialog un-sizable */
    setResizable(false);

    /* Center the dialog */
    Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
    Rectangle frameDim = getBounds();
    setLocation((screenDim.width - frameDim.width) / 2,(screenDim.height - frameDim.height) / 2);

    accessTempFiles(true);

    /* Show the dialog */
    setVisible(true);
  }

  private void layoutCCenterPanel()
  {
   /* Set the C center panel layout and border */
   centerC.setLayout(new GridLayout(1,1));
   centerC.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Instance/Class Variable(s) (usually private)"),BorderFactory.createEmptyBorder(10,10,10,10)));

   /* Add component to the C center panel */
   centerC.add(variableScrollPane);

   /* Add component to the center panel */
   center.add(centerC);
  }

  public void actionPerformed(ActionEvent evt)
  {
    Object obj = evt.getSource();

    if (obj == save)
    {
     if (accessTempFiles(false))
      dispatchEvent(new WindowEvent(this,WindowEvent.WINDOW_CLOSING));
    }
    else if (obj == cancel)
     dispatchEvent(new WindowEvent(this,WindowEvent.WINDOW_CLOSING));
  }

  private boolean accessTempFiles(boolean readOnly)
  {
    /* Save the data to temp files */
    RandomAccessFile sourceFile = null;
    RandomAccessFile variableFile = null;
    RandomAccessFile primaryKeyFieldFile = null;
    RandomAccessFile cmpFieldFile = null;
    String str = null;
    boolean bReturn = true;

    try
    {
      str = "EJB source";
      sourceFile = new RandomAccessFile(TEMP_FILE_PREFIX + owner.base.getText() + TEMP_SOURCE_SUFFIX,(readOnly ? "r" : "rw"));

      if (readOnly)
      {
        byte b[] = new byte[(int) sourceFile.length()];

        sourceFile.read(b);
        sourceTextArea.setText(new String(b));
      }
      else
      {
        sourceFile.setLength(0);
        sourceFile.write(sourceTextArea.getText().getBytes());
      }

      if ((owner.session.isSelected() && owner.stateful.isSelected()) || owner.entity.isSelected())
      {
        str = "EJB instance/class variable(s)";
        variableFile = new RandomAccessFile(TEMP_FILE_PREFIX + owner.base.getText() + TEMP_FIELDS_SUFFIX,(readOnly ? "r" : "rw"));

        if (readOnly)
        {
          byte b[] = new byte[(int) variableFile.length()];

          variableFile.read(b);
          variableTextArea.setText(new String(b));
        }
        else
        {
          variableFile.setLength(0);
          variableFile.write(variableTextArea.getText().getBytes());
        }
      }

      if (owner.entity.isSelected())
      {
        str = "EJB primary key field(s)";
        primaryKeyFieldFile = new RandomAccessFile(TEMP_FILE_PREFIX + owner.base.getText() + TEMP_PK_FIELDS_SUFFIX,(readOnly ? "r" : "rw"));

        if (readOnly)
        {
          byte b[] = new byte[(int) primaryKeyFieldFile.length()];

          primaryKeyFieldFile.read(b);
          primaryKeyTextArea.setText(new String(b));
        }
        else
        {
          primaryKeyFieldFile.setLength(0);
          primaryKeyFieldFile.write(primaryKeyTextArea.getText().getBytes());
        }

        if (owner.containerManaged.isSelected())
        {
          str = "EJB container-managed persistence field(s)";
          cmpFieldFile = new RandomAccessFile(TEMP_FILE_PREFIX + owner.base.getText() + TEMP_CMP_FIELDS_SUFFIX,(readOnly ? "r" : "rw"));

          if (readOnly)
          {
            byte b[] = new byte[(int) cmpFieldFile.length()];

            cmpFieldFile.read(b);
            cmpTextArea.setText(new String(b));
          }
          else
          {
            cmpFieldFile.setLength(0);
            cmpFieldFile.write(cmpTextArea.getText().getBytes());
          }
        }
      }
    }
    catch(FileNotFoundException fnfe)
    {
      if (!readOnly)
      {
       bReturn = false;
       JOptionPane.showMessageDialog(this,"Could not create temp " + str + " file!","Error",JOptionPane.ERROR_MESSAGE);
      }
    }
    catch(IOException ioe)
    {
      if (!readOnly)
      {
       bReturn = false;
       JOptionPane.showMessageDialog(this,"Could not write to temp " + str + " file!","Error",JOptionPane.ERROR_MESSAGE);
      }
    }
    finally
    {
      try
      {
       if (sourceFile != null)
        sourceFile.close();
      }
      catch(IOException ioe)
      {
      }

      try
      {
       if (variableFile != null)
        variableFile.close();
      }
      catch(IOException ioe)
      {
      }

      try
      {
       if (primaryKeyFieldFile != null)
        primaryKeyFieldFile.close();
      }
      catch(IOException ioe)
      {
      }

      try
      {
       if (cmpFieldFile != null)
        cmpFieldFile.close();
      }
      catch(IOException ioe)
      {
      }
    }

    return bReturn;
  }

  class EnableSaveDocument extends PlainDocument
  {
    public void insertString(int offset, String str, AttributeSet set) throws BadLocationException
    {
     int strLen = str.length();
     int len = getLength();

     super.insertString(offset,str,set);
     updateSaveButton();
    }

    public void remove(int offset, int len) throws BadLocationException
    {
     super.remove(offset,len);
     updateSaveButton();
    }

    public void updateSaveButton()
    {
     int len = variableTextArea.getText().length() + cmpTextArea.getText().length() + primaryKeyTextArea.getText().length() + sourceTextArea.getText().length();

     if (len > 0 && !save.isEnabled())
      save.setEnabled(true);
     else if (len == 0 && save.isEnabled())
      save.setEnabled(false);
    }
  }
}

class ProgressDialog extends JDialog implements ActionListener
{
  JTextArea progressTextArea = new JTextArea();
  JScrollPane progressScrollPane = new JScrollPane(progressTextArea,ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
  JButton ok = new JButton("OK");
  JPanel center = new JPanel();
  JPanel south = new JPanel();

  public ProgressDialog(Frame owner)
  {
    super(owner,"Generate",true);

    /* Components should be added to the container's content pane */
    Container cp = getContentPane();

    /* Add action listeners */
    ok.addActionListener(this);

    /* Add the window listener */
    addWindowListener(new WindowAdapter() {
     public void windowClosing(WindowEvent evt) {
      dispose();}});

    /* Disable the text area */
    progressTextArea.setEnabled(false);

    /* Set center panel layout */
    center.setLayout(new GridLayout(1,1));

    /* Add components to the center panel */
    center.add(progressScrollPane);

    /* Add components to the south panel */
    south.add(ok);

    /* Add panels to the dialog */
    cp.add(BorderLayout.CENTER,center);
    cp.add(BorderLayout.SOUTH,south);

    /* Size the dialog */
    setSize(430,170);

    /* Center the dialog */
    Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
    Rectangle frameDim = getBounds();
    setLocation((screenDim.width - frameDim.width) / 2,(screenDim.height - frameDim.height) / 2);
  }

  public void actionPerformed(ActionEvent evt)
  {
    Object obj = evt.getSource();

    if (obj == ok)
     dispatchEvent(new WindowEvent(this,WindowEvent.WINDOW_CLOSING));
  }

  public void addMessage(String msg)
  {
    progressTextArea.append(msg);
  }
}

class FileClassLoader extends ClassLoader
{
 private String path;

 public Class loadClass(String name, boolean resolve) throws ClassNotFoundException
 {
  /* Determine if class has already been loaded */
  Class c = findLoadedClass(name);

  try
  {
   /* Determine if the class can be loaded from the CLASSPATH */
   if (c == null)
    c = findSystemClass(name);
  }
  catch(ClassNotFoundException cnfe)
  {
   //class is not present in the CLASSPATH
  }

  if (c == null)
   c = loadClassBytes(new File(path + "\\" + name + ".class"));

  return c;
 }

 public Class loadClassBytes(File f) throws ClassNotFoundException
 {
  FileInputStream fis = null;
  byte b[] = null;

  try
  {
   fis = new FileInputStream(f);
   b = new byte[(int) f.length()];

   fis.read(b,0,b.length);
  }
  catch(IOException e)
  {
   throw new ClassNotFoundException("Class could not be loaded!");
  }
  finally
  {
    try
    {
      if (fis != null)
       fis.close();
    }
    catch(IOException ioe)
    {
      //no op
    }
  }

  /* Save the file's path name */
  path = f.getParent();

  return defineClass(null,b,0,b.length);
 }
}

interface EJBSkeletonConstants
{
 public static final String TEMP_FILE_PREFIX = "_tmp";
 public static final String TEMP_SOURCE_SUFFIX = "EJBSource." + TEMP_FILE_PREFIX;
 public static final String TEMP_FIELDS_SUFFIX = "EJBFields." + TEMP_FILE_PREFIX;
 public static final String TEMP_PK_FIELDS_SUFFIX = "EJBPkFields." + TEMP_FILE_PREFIX;
 public static final String TEMP_CMP_FIELDS_SUFFIX = "EJBCmpFields." + TEMP_FILE_PREFIX;
}
